// KKeyEdit v1.1 by N's LAB Copyright (C) 2006-2012 by Nobumichi Kozawa
package setup;

import javax.microedition.lcdui.*;
//import javax.microedition.midlet.MIDlet;
//import javax.microedition.rms.*;

public class KKeyEdit extends Canvas implements CommandListener
{
    private int iWidth, iHeight, ix, iy, ifh, iMargin = 5;
    private String title, explain, note, skey2;
    private Font font, sfont;
    public int key = 0;
    private StringBuffer sbKey;
    private KKJSetup kks;
    private boolean bCtrl;

    public void commandAction(Command command, Displayable screen) {
        if (screen.equals(this)) {
               if (command == kks.exitCommand) kks.sc.setValue(key);
            kks.disp.setCurrent(kks.sc.scForm);
        }
    }

    public KKeyEdit(String t, KKJSetup k) {
        bCtrl = false;
        kks = k;
        font = Font.getFont(Font.FACE_MONOSPACE,Font.STYLE_PLAIN,Font.SIZE_MEDIUM);
        sfont = Font.getFont(Font.FACE_MONOSPACE,Font.STYLE_PLAIN,Font.SIZE_SMALL);
        ifh = font.getHeight();
        sbKey = new StringBuffer();
        skey2 = "";
        title = "キー設定 : " + t;
        explain = "ショートカットとして設定したいキーもしくはCtrlと英字キーの組み合わせを押してください。";
        note = "注意) 設定できるのは一つのキーコードのみです。複数のキーコードを出力するキーもしくはコンビネーションは使えません。";
    }

    public void keyPressed(int k) {
        key = k;
        if (bCtrl) {
            if (97 <= key && key <= 122) key -= 96;
            else if (65 <= key && key <= 90) key -= 64;
        }
        if (-53 == key) {
            bCtrl = true;
        }
        String x = Integer.toHexString(key).toUpperCase();
        if (1 == x.length()) x = "0" + x;
        if (font.stringWidth(sbKey.toString() + x) >= iWidth - iMargin * 4) {
            skey2 = sbKey.toString();
            sbKey.delete(0, sbKey.length());
        }
        sbKey.append(x + " ");
        repaint();
    }

    public void keyReleased(int k) {
        if (-53 == k) bCtrl = false;
    }

    public void paint(Graphics g) {
        iWidth = getWidth();
        iHeight = getHeight();
        ix = iMargin;
        iy = 0;
        g.setColor(0x00EEEEEE);
          g.fillRect(0, 0, iWidth, iHeight);
          g.setFont(font);
          g.setColor(0x00000000);
          drawTextLn(g, font, title);
          drawTextLn(g, sfont, explain);
          g.setFont(font);
          g.setColor(0x00000080);
          g.fillRect(iMargin, iy, iWidth - iMargin * 2, ifh);
          if (0 != key) {
              String keydata = keyName(key);
              int kw = font.stringWidth(keydata);
              int km = (kw < iWidth - iMargin * 2) ? (iWidth - iMargin * 2 - kw) / 2 : 0;
              g.setColor(0x00FFFF00);
              g.drawString(keydata, km, iy, Graphics.LEFT | Graphics.TOP);
          }
          iy += ifh + 2;
          g.setColor(0x00000080);
          g.fillRect(iMargin, iy, iWidth - iMargin * 2, ifh * 2);
          g.setColor(0x00FFFFFF);
          g.drawString(sbKey.toString(), iMargin * 2, iy, Graphics.LEFT | Graphics.TOP);
          g.drawString(skey2, iMargin * 2, iy + ifh, Graphics.LEFT | Graphics.TOP);
          iy += ifh * 2;
        g.setColor(0x00000000);
          drawText(g, sfont, note);
    }

    private void drawTextLn(Graphics g, Font f, String txt) {
        int ifh = f.getHeight();

        drawText(g, f, txt);
        ix = iMargin;
        iy += ifh;
    }

    private void drawText(Graphics g, Font f, String txt) {
        char c;
        int ifw;
        int ifh = f.getHeight();

        g.setFont(f);
        for (int i = 0; i < txt.length(); i++) {
            c = txt.charAt(i);
            ifw = f.charWidth(c);
            if (ix + ifw > iWidth - iMargin) {
                ix = iMargin;
                iy += ifh;
            }
            g.drawChar(c, ix, iy, Graphics.LEFT | Graphics.TOP);
            ix += ifw;
        }
    }

    public static String keyName(int key) {
        String chr = "";
        String x = Integer.toHexString(key).toUpperCase();
        if (1 == x.length()) x = "0" + x;
        if (0x20 < key && key < 0x7F) {
            byte[] ba = new byte[1];
            ba[0] = (byte)key;
            chr = new String(ba);
        } else {
            switch (key) {
                case 0 : chr = "NUL"; break;
                case 1 : chr = "Ctrl+A (SOH)"; break;
                case 2 : chr = "Ctrl+B (STX)"; break;
                case 3 : chr = "Ctrl+C (ETX)"; break;
                case 4 : chr = "Ctrl+D (EOT)"; break;
                case 5 : chr = "Ctrl+E (ENQ)"; break;
                case 6 : chr = "Ctrl+F (ACK)"; break;
                case 7 : chr = "Ctrl+G (BEL)"; break;
                case 8 : chr = "Ctrl+H (BS)"; break;
                case 9 : chr = "Ctrl+I (TAB)"; break;
                case 10 : chr = "Ctrl+J (LF)"; break;
                case 11 : chr = "Ctrl+K (VT)"; break;
                case 12 : chr = "Ctrl+L (F)"; break;
                case 13 : chr = "Ctrl+M (CR)"; break;
                case 14 : chr = "Ctrl+N (SO)"; break;
                case 15 : chr = "Ctrl+O (SI)"; break;
                case 16 : chr = "Ctrl+P (DEL)"; break;
                case 17 : chr = "Ctrl+Q (DC1)"; break;
                case 18 : chr = "Ctrl+R (DC2)"; break;
                case 19 : chr = "Ctrl+S (DC3)"; break;
                case 20 : chr = "Ctrl+T (DC4)"; break;
                case 21 : chr = "Ctrl+U (NAK)"; break;
                case 22 : chr = "Ctrl+V (SYN)"; break;
                case 23 : chr = "Ctrl+W (ETB)"; break;
                case 24 : chr = "Ctrl+X (CAN)"; break;
                case 25 : chr = "Ctrl+Y (EM)"; break;
                case 26 : chr = "Ctrl+Z (SUB)"; break;
                case 27 : chr = "(ESC)"; break;
                case 28 : chr = "(FS)"; break;
                case 29 : chr = "(GS)"; break;
                case 30 : chr = "(RS)"; break;
                case 31 : chr = "(US)"; break;
                case 32 : chr = "(SPACE)"; break;
                case 128 : chr = "(DEL)"; break;
                case -53 : chr = "(Ctrl)"; break;
                case -54 : chr = "(Sym)"; break;
            }
        }
        return x + " (" + Integer.toString(key) + "), " + chr;
    }
}